@@ -16,15 +16,18 @@
 #include <avogadro/elementtranslator.h>
 #include <avogadro/periodictableview.h>
 
+#include <QFileInfo>
 #include <QGLFormat>
 #include <QUndoStack>
 #include <QSettings>
 #include <kdebug.h>
 #include <kfiledialog.h>
+#include <kjob.h>
 #include <kstandarddirs.h>
 #include <kmessagebox.h>
 #include <KLocale>
 #include <knewstuff3/downloaddialog.h>
+#include <kio/job.h>
 
 #include <openbabel2wrapper.h>
 
@@ -63,8 +66,8 @@ MoleculeDialog::MoleculeDialog( QWidget * parent )
 	ui.qualityCombo->setCurrentIndex(2); //default to high quality
 	
 	//default to atom symbols
-	ui.labelsCombo->setCurrentIndex(2);
-	ui.glWidget->setLabels(2);
+	ui.labelsCombo->setCurrentIndex(1);
+	ui.glWidget->setLabels(1);
 	
 	//default to balls-and-sticks
 	ui.styleCombo->setCurrentIndex(0); 
@@ -118,7 +121,9 @@ MoleculeDialog::MoleculeDialog( QWidget * parent )
       this, SLOT(slotAdjustHydrogens()));
   connect(ui.optimizeButton, SIGNAL(clicked()),
       this, SLOT(slotGeometryOptimize()));
-      
+  connect(ui.clearDrawingButton, SIGNAL(clicked()),
+      this, SLOT(clearAllElementsInEditor()));
+
   connect(ui.glWidget->molecule(), SIGNAL(updated()),
           this, SLOT(slotUpdateStatistics()));
 
@@ -174,8 +179,13 @@ void MoleculeDialog::slotLoadMolecule()
           this,
           i18n( "Choose a file to open" ) );
 
-  if( filename.isEmpty() ) return;
+  loadMolecule(filename);
+}
 
+void MoleculeDialog::loadMolecule(const QString &filename)
+{
+  if( filename.isEmpty() ) return;
+  
   kDebug() << "Filename to load: " << filename;
 
   Molecule* molecule = OpenBabel2Wrapper::readMolecule( filename );
@@ -196,6 +206,11 @@ void MoleculeDialog::slotLoadMolecule()
   ui.glWidget->invalidateDLs();
 }
 
+void MoleculeDialog::clearAllElementsInEditor(){
+  ui.glWidget->molecule()->clear();
+  ui.glWidget->update();
+}
+
 void MoleculeDialog::slotSaveMolecule()
 {
   KUrl url("kfiledialog:///kalzium");
@@ -252,10 +267,44 @@ void MoleculeDialog::slotDownloadNewStuff()
     dialog.exec();
 
     // list of changed entries
+    QString destinationDir = KGlobalSettings::documentPath();
+    QDir dir(destinationDir);
+    if (!dir.exists())
+        destinationDir = QDir::homePath();
+    bool anyError = false;
+    bool anySuccess = false;
+    bool moreOneInstalledFile = false;
+    QString exactlyOneFile;
     foreach (const KNS3::Entry& entry, dialog.changedEntries()) {
         // care only about installed ones
         if (entry.status() == KNS3::Entry::Installed) {
             kDebug() << "Changed Entry: " << entry.installedFiles();
+            foreach (const QString &origFile, entry.installedFiles()) {
+                const QString destFile = destinationDir + '/' + QFileInfo(origFile).fileName();
+                KJob *job = KIO::file_move(KUrl::fromPath(origFile), KUrl::fromPath(destFile));
+                const bool success = job->exec();
+                if (success) {
+                    if (exactlyOneFile.isEmpty()) {
+                        exactlyOneFile = destFile;
+                    } else {
+                        moreOneInstalledFile = true;
+                    }
+                    anySuccess = true;
+                } else {
+                    KMessageBox::error(this, i18n("Failed to download molecule %1 to %2.", entry.name(), destFile));
+                    anyError = true;
+                }
+            }
+        }
+    }
+    if (anySuccess) {
+        if (anyError) {
+            KMessageBox::information(this, i18n("The molecules that could be downloaded have been saved to %1.", destinationDir));
+        } else {
+            KMessageBox::information(this, i18n("The molecules have been saved to %1.", destinationDir));
+        }
+        if (!moreOneInstalledFile) {
+            loadMolecule(exactlyOneFile);
         }
     }
 }
